@@ -730,6 +730,10 @@ gdk_event_get_root_coords (GdkEvent *event,
       x = event->motion.x_root;
       y = event->motion.y_root;
       break;
+    case GDK_SCROLL:
+      x = event->scroll.x_root;
+      y = event->scroll.y_root;
+      break;
     case GDK_BUTTON_PRESS:
     case GDK_2BUTTON_PRESS:
     case GDK_3BUTTON_PRESS:
@@ -1042,7 +1046,7 @@ _gdk_event_button_generate (GdkDisplay *display,
       display->button_click_time[1] = 0;
       display->button_click_time[0] = 0;
       display->button_window[1] = NULL;
-      display->button_window[0] = 0;
+      display->button_window[0] = NULL;
       display->button_number[1] = -1;
       display->button_number[0] = -1;
       display->button_x[0] = display->button_x[1] = 0;
