@@ -27,12 +27,13 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <string.h>
 #include <stdarg.h>
 #include <gdkkeysyms.h>
+#include "gtkalias.h"
 #include "gtkbindings.h"
 #include "gtkkeyhash.h"
-#include "gtksignal.h"
 #include "gtkwidget.h"
 #include "gtkrc.h"
 
@@ -80,7 +81,7 @@ binding_signal_free (GtkBindingSignal *sig)
   
   for (i = 0; i < sig->n_args; i++)
     {
-      if (GTK_FUNDAMENTAL_TYPE (sig->args[i].arg_type) == GTK_TYPE_STRING)
+      if (G_TYPE_FUNDAMENTAL (sig->args[i].arg_type) == G_TYPE_STRING)
 	g_free (sig->args[i].d.string_data);
     }
   g_free (sig->args);
@@ -362,11 +363,11 @@ binding_compose_params (GtkObject       *object,
 	  g_value_init (&tmp_value, G_TYPE_DOUBLE);
 	  g_value_set_double (&tmp_value, args->d.double_data);
 	  break;
-	case  G_TYPE_LONG:
+	case G_TYPE_LONG:
 	  g_value_init (&tmp_value, G_TYPE_LONG);
 	  g_value_set_long (&tmp_value, args->d.long_data);
 	  break;
-	case  G_TYPE_STRING:
+	case G_TYPE_STRING:
 	  /* gtk_rc_parse_flags/enum() has fancier parsing for this; we can't call
 	   * that since we don't have a GParamSpec, so just do something simple
 	   */
@@ -505,7 +506,7 @@ gtk_binding_entry_activate (GtkBindingEntry *entry,
 		     sig->signal_name,
 		     g_type_name (G_OBJECT_TYPE (object)));
 	}
-      else if (!(query.signal_flags & GTK_RUN_ACTION))
+      else if (!(query.signal_flags & G_SIGNAL_ACTION))
 	{
 	  accelerator = gtk_accelerator_name (entry->keyval, entry->modifiers);
 	  g_warning ("gtk_binding_entry_activate(): binding \"%s::%s\": "
@@ -564,6 +565,7 @@ gtk_binding_set_new (const gchar    *set_name)
   binding_set->class_branch_pspecs = NULL;
   binding_set->entries = NULL;
   binding_set->current = NULL;
+  binding_set->parsed = FALSE;
   
   binding_set_list = g_slist_prepend (binding_set_list, binding_set);
   
@@ -586,10 +588,10 @@ gtk_binding_set_by_class (gpointer object_class)
   if (binding_set)
     return binding_set;
 
-  binding_set = gtk_binding_set_new (gtk_type_name (GTK_CLASS_TYPE (class)));
+  binding_set = gtk_binding_set_new (g_type_name (G_OBJECT_CLASS_TYPE (class)));
   gtk_binding_set_add_path (binding_set,
 			    GTK_PATH_CLASS,
-			    gtk_type_name (GTK_CLASS_TYPE (class)),
+			    g_type_name (G_OBJECT_CLASS_TYPE (class)),
 			    GTK_PATH_PRIO_GTK);
   g_dataset_id_set_data (class, key_id_class_binding_set, binding_set);
 
@@ -704,19 +706,19 @@ gtk_binding_entry_add_signall (GtkBindingSet  *binding_set,
 	  binding_signal_free (signal);
 	  return;
 	}
-      switch (GTK_FUNDAMENTAL_TYPE (tmp_arg->arg_type))
+      switch (G_TYPE_FUNDAMENTAL (tmp_arg->arg_type))
 	{
-	case  GTK_TYPE_LONG:
-	  arg->arg_type = GTK_TYPE_LONG;
+	case  G_TYPE_LONG:
+	  arg->arg_type = G_TYPE_LONG;
 	  arg->d.long_data = tmp_arg->d.long_data;
 	  break;
-	case  GTK_TYPE_DOUBLE:
-	  arg->arg_type = GTK_TYPE_DOUBLE;
+	case  G_TYPE_DOUBLE:
+	  arg->arg_type = G_TYPE_DOUBLE;
 	  arg->d.double_data = tmp_arg->d.double_data;
 	  break;
-	case  GTK_TYPE_STRING:
+	case  G_TYPE_STRING:
           if (tmp_arg->arg_type != GTK_TYPE_IDENTIFIER)
-	    arg->arg_type = GTK_TYPE_STRING;
+	    arg->arg_type = G_TYPE_STRING;
 	  else
 	    arg->arg_type = GTK_TYPE_IDENTIFIER;
 	  arg->d.string_data = g_strdup (tmp_arg->d.string_data);
@@ -729,7 +731,7 @@ gtk_binding_entry_add_signall (GtkBindingSet  *binding_set,
 	  break;
 	default:
 	  g_warning ("gtk_binding_entry_add_signall(): unsupported type `%s' for arg[%u]",
-		     gtk_type_name (arg->arg_type), n);
+		     g_type_name (arg->arg_type), n);
 	  binding_signal_free (signal);
 	  return;
 	}
@@ -774,46 +776,46 @@ gtk_binding_entry_add_signal (GtkBindingSet  *binding_set,
       slist = g_slist_prepend (slist, arg);
 
       arg->arg_type = va_arg (args, GtkType);
-      switch (GTK_FUNDAMENTAL_TYPE (arg->arg_type))
+      switch (G_TYPE_FUNDAMENTAL (arg->arg_type))
 	{
 	  /* for elaborated commenting about var args collection, take a look
 	   * at gtk_arg_collect_value() in gtkargcollector.c
 	   */
-	case GTK_TYPE_CHAR:
-	case GTK_TYPE_UCHAR:
-	case GTK_TYPE_INT:
-	case GTK_TYPE_UINT:
-	case GTK_TYPE_BOOL:
-	case GTK_TYPE_ENUM:
-	case GTK_TYPE_FLAGS:
-	  arg->arg_type = GTK_TYPE_LONG;
+	case G_TYPE_CHAR:
+	case G_TYPE_UCHAR:
+	case G_TYPE_INT:
+	case G_TYPE_UINT:
+	case G_TYPE_BOOLEAN:
+	case G_TYPE_ENUM:
+	case G_TYPE_FLAGS:
+	  arg->arg_type = G_TYPE_LONG;
 	  arg->d.long_data = va_arg (args, gint);
 	  break;
-	case GTK_TYPE_LONG:
-	case GTK_TYPE_ULONG:
-	  arg->arg_type = GTK_TYPE_LONG;
+	case G_TYPE_LONG:
+	case G_TYPE_ULONG:
+	  arg->arg_type = G_TYPE_LONG;
 	  arg->d.long_data = va_arg (args, glong);
 	  break;
-	case GTK_TYPE_FLOAT:
-	case GTK_TYPE_DOUBLE:
-	  arg->arg_type = GTK_TYPE_DOUBLE;
+	case G_TYPE_FLOAT:
+	case G_TYPE_DOUBLE:
+	  arg->arg_type = G_TYPE_DOUBLE;
 	  arg->d.double_data = va_arg (args, gdouble);
 	  break;
-	case GTK_TYPE_STRING:
+	case G_TYPE_STRING:
 	  if (arg->arg_type != GTK_TYPE_IDENTIFIER)
-	    arg->arg_type = GTK_TYPE_STRING;
+	    arg->arg_type = G_TYPE_STRING;
 	  arg->d.string_data = va_arg (args, gchar*);
 	  if (!arg->d.string_data)
 	    {
 	      g_warning ("gtk_binding_entry_add_signal(): type `%s' arg[%u] is `NULL'",
-			 gtk_type_name (arg->arg_type),
+			 g_type_name (arg->arg_type),
 			 i);
 	      i += n_args + 1;
 	    }
 	  break;
 	default:
 	  g_warning ("gtk_binding_entry_add_signal(): unsupported type `%s' for arg[%u]",
-		     gtk_type_name (arg->arg_type), i);
+		     g_type_name (arg->arg_type), i);
 	  i += n_args + 1;
 	  break;
 	}
@@ -1045,24 +1047,24 @@ gtk_bindings_activate_list (GtkObject *object,
   if (!handled)
     {
       GSList *patterns;
-      GtkType class_type;
+      GType class_type;
       
       patterns = gtk_binding_entries_sort_patterns (entries, GTK_PATH_CLASS, is_release);
-      class_type = GTK_OBJECT_TYPE (object);
+      class_type = G_TYPE_FROM_INSTANCE (object);
       while (class_type && !handled)
 	{
 	  guint path_length;
 	  const gchar *path;
 	  gchar *path_reversed;
 	  
-	  path = gtk_type_name (class_type);
+	  path = g_type_name (class_type);
 	  path_reversed = g_strdup (path);
 	  g_strreverse (path_reversed);
 	  path_length = strlen (path);
 	  handled = binding_match_activate (patterns, object, path_length, path, path_reversed);
 	  g_free (path_reversed);
 
-	  class_type = gtk_type_parent (class_type);
+	  class_type = g_type_parent (class_type);
 	}
       g_slist_free (patterns);
     }
@@ -1076,6 +1078,7 @@ gtk_bindings_activate (GtkObject      *object,
 		       GdkModifierType modifiers)
 {
   GSList *entries = NULL;
+  GdkDisplay *display;
   GtkKeyHash *key_hash;
   gboolean handled = FALSE;
   gboolean is_release;
@@ -1088,7 +1091,9 @@ gtk_bindings_activate (GtkObject      *object,
   is_release = (modifiers & GDK_RELEASE_MASK) != 0;
   modifiers = modifiers & BINDING_MOD_MASK () & ~GDK_RELEASE_MASK;
 
-  key_hash = binding_key_hash_for_keymap (gdk_keymap_get_default ());
+  display = gtk_widget_get_display (GTK_WIDGET (object));
+  key_hash = binding_key_hash_for_keymap (gdk_keymap_get_for_display (display));
+  
   entries = _gtk_key_hash_lookup_keyval (key_hash, keyval, modifiers);
 
   handled = gtk_bindings_activate_list (object, entries, is_release);
@@ -1099,7 +1104,7 @@ gtk_bindings_activate (GtkObject      *object,
 }
 
 /**
- * _gtk_bindings_activate_event:
+ * gtk_bindings_activate_event:
  * @object: a #GtkObject (generally must be a widget)
  * @event: a #GdkEventKey
  * 
@@ -1109,10 +1114,11 @@ gtk_bindings_activate (GtkObject      *object,
  * Return value: %TRUE if a matching key binding was found
  **/
 gboolean
-_gtk_bindings_activate_event (GtkObject      *object,
-			      GdkEventKey    *event)
+gtk_bindings_activate_event (GtkObject      *object,
+                             GdkEventKey    *event)
 {
   GSList *entries = NULL;
+  GdkDisplay *display;
   GtkKeyHash *key_hash;
   gboolean handled = FALSE;
 
@@ -1121,10 +1127,13 @@ _gtk_bindings_activate_event (GtkObject      *object,
   if (!GTK_IS_WIDGET (object))
     return FALSE;
 
-  key_hash = binding_key_hash_for_keymap (gdk_keymap_get_default ());
+  display = gtk_widget_get_display (GTK_WIDGET (object));
+  key_hash = binding_key_hash_for_keymap (gdk_keymap_get_for_display (display));
+
   entries = _gtk_key_hash_lookup (key_hash,
 				  event->hardware_keycode,
-				  event->state & BINDING_MOD_MASK () & ~GDK_RELEASE_MASK,
+				  event->state,
+				  BINDING_MOD_MASK () & ~GDK_RELEASE_MASK,
 				  event->group);
   
   handled = gtk_bindings_activate_list (object, entries,
@@ -1186,7 +1195,7 @@ gtk_binding_parse_signal (GScanner       *scanner,
 	    {
 	      need_arg = FALSE;
 	      arg = g_new (GtkBindingArg, 1);
-	      arg->arg_type = GTK_TYPE_DOUBLE;
+	      arg->arg_type = G_TYPE_DOUBLE;
 	      arg->d.double_data = scanner->value.v_float;
 	      if (negate)
 		{
@@ -1203,7 +1212,7 @@ gtk_binding_parse_signal (GScanner       *scanner,
 	    {
 	      need_arg = FALSE;
 	      arg = g_new (GtkBindingArg, 1);
-	      arg->arg_type = GTK_TYPE_LONG;
+	      arg->arg_type = G_TYPE_LONG;
 	      arg->d.long_data = scanner->value.v_int;
 	      if (negate)
 		{
@@ -1220,7 +1229,7 @@ gtk_binding_parse_signal (GScanner       *scanner,
 	    {
 	      need_arg = FALSE;
 	      arg = g_new (GtkBindingArg, 1);
-	      arg->arg_type = GTK_TYPE_STRING;
+	      arg->arg_type = G_TYPE_STRING;
 	      arg->d.string_data = g_strdup (scanner->value.v_string);
 	      args = g_slist_prepend (args, arg);
 	    }
@@ -1283,7 +1292,7 @@ gtk_binding_parse_signal (GScanner       *scanner,
       GtkBindingArg *arg;
 
       arg = slist->data;
-      if (GTK_FUNDAMENTAL_TYPE (arg->arg_type) == GTK_TYPE_STRING)
+      if (G_TYPE_FUNDAMENTAL (arg->arg_type) == G_TYPE_STRING)
 	g_free (arg->d.string_data);
       g_free (arg);
     }
@@ -1466,3 +1475,32 @@ _gtk_binding_reset_parsed (void)
       slist = next;
     }
 }
+
+guint
+_gtk_binding_signal_new (const gchar	    *signal_name,
+			 GType		     itype,
+			 GSignalFlags	     signal_flags,
+			 GCallback           handler,
+			 GSignalAccumulator  accumulator,
+			 gpointer	     accu_data,
+			 GSignalCMarshaller  c_marshaller,
+			 GType		     return_type,
+			 guint		     n_params,
+			 ...)
+{
+  va_list args;
+  guint signal_id;
+
+  g_return_val_if_fail (signal_name != NULL, 0);
+  
+  va_start (args, n_params);
+
+  signal_id = g_signal_new_valist (signal_name, itype, signal_flags,
+                                   g_cclosure_new (handler, NULL, NULL),
+				   accumulator, accu_data, c_marshaller,
+                                   return_type, n_params, args);
+
+  va_end (args);
+ 
+  return signal_id;
+}
