@@ -24,11 +24,12 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include "gtkalias.h"
 #include "gtkhscale.h"
-#include "gtksignal.h"
 #include "gtkintl.h"
 
 static gpointer parent_class;
@@ -38,26 +39,32 @@ static void     gtk_hscale_init             (GtkHScale      *hscale);
 static gboolean gtk_hscale_expose           (GtkWidget      *widget,
                                              GdkEventExpose *event);
 
-GtkType
+static void     gtk_hscale_get_layout_offsets (GtkScale       *scale,
+                                               gint           *x,
+                                               gint           *y);
+
+GType
 gtk_hscale_get_type (void)
 {
-  static GtkType hscale_type = 0;
+  static GType hscale_type = 0;
   
   if (!hscale_type)
     {
-      static const GtkTypeInfo hscale_info =
+      static const GTypeInfo hscale_info =
       {
-        "GtkHScale",
-        sizeof (GtkHScale),
         sizeof (GtkHScaleClass),
-        (GtkClassInitFunc) gtk_hscale_class_init,
-        (GtkObjectInitFunc) gtk_hscale_init,
-        /* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+        (GClassInitFunc) gtk_hscale_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+        sizeof (GtkHScale),
+	0,		/* n_preallocs */
+        (GInstanceInitFunc) gtk_hscale_init,
       };
       
-      hscale_type = gtk_type_unique (GTK_TYPE_SCALE, &hscale_info);
+      hscale_type = g_type_register_static (GTK_TYPE_SCALE, "GtkHScale",
+					    &hscale_info, 0);
     }
   
   return hscale_type;
@@ -66,17 +73,19 @@ gtk_hscale_get_type (void)
 static void
 gtk_hscale_class_init (GtkHScaleClass *class)
 {
-  GObjectClass *gobject_class;
   GtkWidgetClass *widget_class;
   GtkRangeClass *range_class;
+  GtkScaleClass *scale_class;
   
-  gobject_class = G_OBJECT_CLASS (class);
   widget_class = GTK_WIDGET_CLASS (class);
   range_class = GTK_RANGE_CLASS (class);
+  scale_class = GTK_SCALE_CLASS (class);
 
   parent_class = g_type_class_peek_parent (class);
 
   range_class->slider_detail = "hscale";
+
+  scale_class->get_layout_offsets = gtk_hscale_get_layout_offsets;
   
   widget_class->expose_event = gtk_hscale_expose;
 }
@@ -95,13 +104,7 @@ gtk_hscale_init (GtkHScale *hscale)
 GtkWidget*
 gtk_hscale_new (GtkAdjustment *adjustment)
 {
-  GtkWidget *hscale;
-  
-  hscale = gtk_widget_new (GTK_TYPE_HSCALE,
-			   "adjustment", adjustment,
-			   NULL);
-
-  return hscale;
+  return g_object_new (GTK_TYPE_HSCALE, "adjustment", adjustment, NULL);
 }
 
 /**
@@ -152,69 +155,25 @@ static gboolean
 gtk_hscale_expose (GtkWidget      *widget,
                    GdkEventExpose *event)
 {
-  GtkRange *range;
-  GtkHScale *hscale;
   GtkScale *scale;
   
-  range = GTK_RANGE (widget);
   scale = GTK_SCALE (widget);
-  hscale = GTK_HSCALE (widget);
-  
+
+  /* We need to chain up _first_ so the various geometry members of
+   * GtkRange struct are updated.
+   */
+  if (GTK_WIDGET_CLASS (parent_class)->expose_event)
+    GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+
   if (scale->draw_value)
     {
       PangoLayout *layout;
-      PangoRectangle logical_rect;
-      gchar *txt;
       gint x, y;
       GtkStateType state_type;
-      gint value_spacing;
 
-      gtk_widget_style_get (widget, "value_spacing", &value_spacing, NULL);
-      
-      txt = _gtk_scale_format_value (scale,
-                                     GTK_RANGE (scale)->adjustment->value);
-      
-      layout = gtk_widget_create_pango_layout (widget, txt);
-      g_free (txt);
-      
-      pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
-      
-      switch (scale->value_pos)
-        {
-        case GTK_POS_LEFT:
-          x = range->range_rect.x - value_spacing - logical_rect.width;
-          y = range->range_rect.y + (range->range_rect.height - logical_rect.height) / 2;
-          break;
-          
-        case GTK_POS_RIGHT:
-          x = range->range_rect.x + range->range_rect.width + value_spacing;
-          y = range->range_rect.y + (range->range_rect.height - logical_rect.height) / 2;
-          break;
-          
-        case GTK_POS_TOP:
-          x = range->slider_start +
-            (range->slider_end - range->slider_start - logical_rect.width) / 2;
-          x = CLAMP (x, 0, widget->allocation.width - logical_rect.width);
-          y = range->range_rect.y - logical_rect.height - value_spacing;
-          break;
-          
-        case GTK_POS_BOTTOM:
-          x = range->slider_start +
-            (range->slider_end - range->slider_start - logical_rect.width) / 2;
-          x = CLAMP (x, 0, widget->allocation.width - logical_rect.width);
-          y = range->range_rect.y + range->range_rect.height + value_spacing;
-          break;
+      layout = gtk_scale_get_layout (scale);
+      gtk_scale_get_layout_offsets (scale, &x, &y);
 
-        default:
-          g_return_val_if_reached (FALSE);
-          x = 0;
-          y = 0;
-          break;
-        }
-
-      x += widget->allocation.x;
-      y += widget->allocation.y;
-      
       state_type = GTK_STATE_NORMAL;
       if (!GTK_WIDGET_IS_SENSITIVE (scale))
         state_type = GTK_STATE_INSENSITIVE;
@@ -229,8 +188,73 @@ gtk_hscale_expose (GtkWidget      *widget,
                         x, y,
                         layout);
 
-      g_object_unref (G_OBJECT (layout));
+    }
+  
+  return FALSE;
+}
+
+static void     
+gtk_hscale_get_layout_offsets (GtkScale *scale,
+                               gint     *x,
+                               gint     *y)
+{
+  GtkWidget *widget;
+  GtkRange *range;
+  PangoLayout *layout;
+  PangoRectangle logical_rect;
+  gint value_spacing;
+
+  widget = GTK_WIDGET (scale);
+  layout = gtk_scale_get_layout (scale);
+      
+  if (!layout)    
+    {
+      *x = 0;
+      *y = 0;
+
+      return;
     }
 
-  return (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
+  gtk_widget_style_get (widget, "value_spacing", &value_spacing, NULL);
+
+  range = GTK_RANGE (widget);
+  scale = GTK_SCALE (widget);
+
+  pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+      
+  switch (scale->value_pos)
+    {
+     case GTK_POS_LEFT:
+       *x = range->range_rect.x - value_spacing - logical_rect.width;
+       *y = range->range_rect.y + (range->range_rect.height - logical_rect.height) / 2;
+       break;
+          
+    case GTK_POS_RIGHT:
+      *x = range->range_rect.x + range->range_rect.width + value_spacing;
+      *y = range->range_rect.y + (range->range_rect.height - logical_rect.height) / 2;
+      break;
+          
+    case GTK_POS_TOP:
+      *x = range->slider_start +
+        (range->slider_end - range->slider_start - logical_rect.width) / 2;
+      *x = CLAMP (*x, 0, widget->allocation.width - logical_rect.width);
+      *y = range->range_rect.y - logical_rect.height - value_spacing;
+      break;
+          
+    case GTK_POS_BOTTOM:
+      *x = range->slider_start +
+        (range->slider_end - range->slider_start - logical_rect.width) / 2;
+      *x = CLAMP (*x, 0, widget->allocation.width - logical_rect.width);
+      *y = range->range_rect.y + range->range_rect.height + value_spacing;
+      break;
+
+    default:
+      g_return_if_reached ();
+      *x = 0;
+      *y = 0;
+      break;
+    }
+
+  *x += widget->allocation.x;
+  *y += widget->allocation.y;
 }
