@@ -27,8 +27,9 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include	"config.h"
+#include	<config.h>
 
+#include 	"gtkalias.h"
 #include	"gtkitemfactory.h"
 #include	"gtk/gtkmenubar.h"
 #include	"gtk/gtkmenu.h"
