@@ -1,4 +1,4 @@
-/* gtkpathbar.h
+/* gtkpathbar.c
  * Copyright (C) 2004  Red Hat, Inc.,  Jonathan Blandford <jrb@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -19,9 +19,12 @@
 
 #include <config.h>
 #include <string.h>
+#include "gtkalias.h"
 #include "gtkpathbar.h"
 #include "gtktogglebutton.h"
+#include "gtkalignment.h"
 #include "gtkarrow.h"
+#include "gtkdnd.h"
 #include "gtkimage.h"
 #include "gtkintl.h"
 #include "gtkicontheme.h"
@@ -45,6 +48,9 @@ typedef enum {
 
 #define BUTTON_DATA(x) ((ButtonData *)(x))
 
+#define SCROLL_TIMEOUT           150
+#define INITIAL_SCROLL_TIMEOUT   300
+
 static guint path_bar_signals [LAST_SIGNAL] = { 0 };
 
 /* Icon size for if we can't get it from the theme */
@@ -60,7 +66,8 @@ struct _ButtonData
   GtkFilePath *path;
   GtkWidget *image;
   GtkWidget *label;
-  gboolean ignore_changes;
+  guint ignore_changes : 1;
+  guint file_is_hidden : 1;
 };
 
 G_DEFINE_TYPE (GtkPathBar,
@@ -85,6 +92,16 @@ static void gtk_path_bar_scroll_up                (GtkWidget        *button,
 						   GtkPathBar       *path_bar);
 static void gtk_path_bar_scroll_down              (GtkWidget        *button,
 						   GtkPathBar       *path_bar);
+static gboolean gtk_path_bar_slider_button_press  (GtkWidget        *widget,
+						   GdkEventButton   *event,
+						   GtkPathBar       *path_bar);
+static gboolean gtk_path_bar_slider_button_release(GtkWidget        *widget,
+						   GdkEventButton   *event,
+						   GtkPathBar       *path_bar);
+static void gtk_path_bar_grab_notify              (GtkWidget        *widget,
+						   gboolean          was_grabbed);
+static void gtk_path_bar_state_changed            (GtkWidget        *widget,
+						   GtkStateType      previous_state);
 static void gtk_path_bar_style_set                (GtkWidget        *widget,
 						   GtkStyle         *previous_style);
 static void gtk_path_bar_screen_changed           (GtkWidget        *widget,
@@ -122,9 +139,14 @@ gtk_path_bar_init (GtkPathBar *path_bar)
   path_bar->up_slider_button = get_slider_button (path_bar, GTK_ARROW_LEFT);
   path_bar->down_slider_button = get_slider_button (path_bar, GTK_ARROW_RIGHT);
   path_bar->icon_size = FALLBACK_ICON_SIZE;
-
+  
   g_signal_connect (path_bar->up_slider_button, "clicked", G_CALLBACK (gtk_path_bar_scroll_up), path_bar);
   g_signal_connect (path_bar->down_slider_button, "clicked", G_CALLBACK (gtk_path_bar_scroll_down), path_bar);
+
+  g_signal_connect (path_bar->up_slider_button, "button_press_event", G_CALLBACK (gtk_path_bar_slider_button_press), path_bar);
+  g_signal_connect (path_bar->up_slider_button, "button_release_event", G_CALLBACK (gtk_path_bar_slider_button_release), path_bar);
+  g_signal_connect (path_bar->down_slider_button, "button_press_event", G_CALLBACK (gtk_path_bar_slider_button_press), path_bar);
+  g_signal_connect (path_bar->down_slider_button, "button_release_event", G_CALLBACK (gtk_path_bar_slider_button_release), path_bar);
 }
 
 static void
@@ -147,6 +169,8 @@ gtk_path_bar_class_init (GtkPathBarClass *path_bar_class)
   widget_class->size_allocate = gtk_path_bar_size_allocate;
   widget_class->style_set = gtk_path_bar_style_set;
   widget_class->screen_changed = gtk_path_bar_screen_changed;
+  widget_class->grab_notify = gtk_path_bar_grab_notify;
+  widget_class->state_changed = gtk_path_bar_state_changed;
 
   container_class->add = gtk_path_bar_add;
   container_class->forall = gtk_path_bar_forall;
@@ -155,14 +179,15 @@ gtk_path_bar_class_init (GtkPathBarClass *path_bar_class)
   /*  container_class->child_type = gtk_path_bar_child_type;*/
 
   path_bar_signals [PATH_CLICKED] =
-    g_signal_new ("path_clicked",
+    g_signal_new ("path-clicked",
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkPathBarClass, path_clicked),
 		  NULL, NULL,
-		  _gtk_marshal_VOID__POINTER,
-		  G_TYPE_NONE, 1,
-		  G_TYPE_POINTER);
+		  _gtk_marshal_VOID__POINTER_BOOLEAN,
+		  G_TYPE_NONE, 2,
+		  G_TYPE_POINTER,
+		  G_TYPE_BOOLEAN);
 }
 
 
@@ -249,7 +274,7 @@ gtk_path_bar_size_request (GtkWidget      *widget,
   /* Theoretically, the slider could be bigger than the other button.  But we're
    * not going to worry about that now.
    */
-  path_bar->slider_width = requisition->height / 2 + 5;
+  path_bar->slider_width = MIN(requisition->height * 2 / 3 + 5, requisition->height);
   if (path_bar->button_list && path_bar->button_list->next != NULL)
     requisition->width += (path_bar->spacing + path_bar->slider_width) * 2;
 
@@ -424,11 +449,13 @@ gtk_path_bar_size_allocate (GtkWidget     *widget,
 	{
 	  child_allocation.x -= path_bar->spacing;
 	  down_slider_offset = child_allocation.x - widget->allocation.x - path_bar->slider_width;
+	  down_slider_offset = border_width;
 	}
       else
 	{
-	  child_allocation.x += child_allocation.width + path_bar->spacing;
 	  down_slider_offset = child_allocation.x - widget->allocation.x;
+	  down_slider_offset = allocation->width - border_width - path_bar->slider_width;
+	  child_allocation.x += child_allocation.width + path_bar->spacing;
 	}
     }
   /* Now we go hide all the widgets that don't fit */
@@ -496,6 +523,16 @@ gtk_path_bar_add (GtkContainer *container,
   gtk_widget_set_parent (widget, GTK_WIDGET (container));
 }
 
+static void
+gtk_path_bar_remove_1 (GtkContainer *container,
+		       GtkWidget    *widget)
+{
+  gboolean was_visible = GTK_WIDGET_VISIBLE (widget);
+  gtk_widget_unparent (widget);
+  if (was_visible)
+    gtk_widget_queue_resize (GTK_WIDGET (container));
+}
+
 static void
 gtk_path_bar_remove (GtkContainer *container,
 		     GtkWidget    *widget)
@@ -505,23 +542,29 @@ gtk_path_bar_remove (GtkContainer *container,
 
   path_bar = GTK_PATH_BAR (container);
 
-  children = path_bar->button_list;
+  if (widget == path_bar->up_slider_button)
+    {
+      gtk_path_bar_remove_1 (container, widget);
+      path_bar->up_slider_button = NULL;
+      return;
+    }
 
+  if (widget == path_bar->down_slider_button)
+    {
+      gtk_path_bar_remove_1 (container, widget);
+      path_bar->down_slider_button = NULL;
+      return;
+    }
+
+  children = path_bar->button_list;
   while (children)
     {
       if (widget == BUTTON_DATA (children->data)->button)
 	{
-	  gboolean was_visible;
-
-	  was_visible = GTK_WIDGET_VISIBLE (widget);
-	  gtk_widget_unparent (widget);
-
+	  gtk_path_bar_remove_1 (container, widget);
 	  path_bar->button_list = g_list_remove_link (path_bar->button_list, children);
 	  g_list_free (children);
-
-	  if (was_visible)
-	    gtk_widget_queue_resize (GTK_WIDGET (container));
-	  break;
+	  return;
 	}
       
       children = children->next;
@@ -550,8 +593,11 @@ gtk_path_bar_forall (GtkContainer *container,
       (* callback) (child, callback_data);
     }
 
-  (* callback) (path_bar->up_slider_button, callback_data);
-  (* callback) (path_bar->down_slider_button, callback_data);
+  if (path_bar->up_slider_button)
+    (* callback) (path_bar->up_slider_button, callback_data);
+
+  if (path_bar->down_slider_button)
+    (* callback) (path_bar->down_slider_button, callback_data);
 }
 
 static void
@@ -564,7 +610,13 @@ gtk_path_bar_scroll_down (GtkWidget *button, GtkPathBar *path_bar)
   gint space_needed;
   gint border_width;
   GtkTextDirection direction;
-  
+
+  if (path_bar->ignore_click)
+    {
+      path_bar->ignore_click = FALSE;
+      return;   
+    }
+
   gtk_widget_queue_resize (GTK_WIDGET (path_bar));
 
   border_width = GTK_CONTAINER (path_bar)->border_width;
@@ -616,6 +668,12 @@ gtk_path_bar_scroll_up (GtkWidget *button, GtkPathBar *path_bar)
 {
   GList *list;
 
+  if (path_bar->ignore_click)
+    {
+      path_bar->ignore_click = FALSE;
+      return;   
+    }
+
   gtk_widget_queue_resize (GTK_WIDGET (path_bar));
 
   for (list = g_list_last (path_bar->button_list); list; list = list->prev)
@@ -628,6 +686,110 @@ gtk_path_bar_scroll_up (GtkWidget *button, GtkPathBar *path_bar)
     }
 }
 
+static gboolean
+gtk_path_bar_scroll_timeout (GtkPathBar *path_bar)
+{
+  gboolean retval = FALSE;
+
+  GDK_THREADS_ENTER ();
+
+  if (path_bar->timer)
+    {
+      if (GTK_WIDGET_HAS_FOCUS (path_bar->up_slider_button))
+	gtk_path_bar_scroll_up (path_bar->up_slider_button, path_bar);
+      else if (GTK_WIDGET_HAS_FOCUS (path_bar->down_slider_button))
+	gtk_path_bar_scroll_down (path_bar->down_slider_button, path_bar);
+
+      if (path_bar->need_timer) 
+	{
+	  path_bar->need_timer = FALSE;
+
+	  path_bar->timer = g_timeout_add (SCROLL_TIMEOUT,
+					   (GSourceFunc)gtk_path_bar_scroll_timeout,
+					   path_bar);
+	  
+	}
+      else
+	retval = TRUE;
+      
+    }
+
+  GDK_THREADS_LEAVE ();
+
+  return retval;
+}
+
+static void 
+gtk_path_bar_stop_scrolling (GtkPathBar *path_bar)
+{
+  if (path_bar->timer)
+    {
+      g_source_remove (path_bar->timer);
+      path_bar->timer = 0;
+      path_bar->need_timer = FALSE;
+    }
+}
+
+static gboolean
+gtk_path_bar_slider_button_press (GtkWidget      *widget, 
+				  GdkEventButton *event,
+				  GtkPathBar     *path_bar)
+{
+  if (!GTK_WIDGET_HAS_FOCUS (widget))
+    gtk_widget_grab_focus (widget);
+
+  if (event->type != GDK_BUTTON_PRESS || event->button != 1)
+    return FALSE;
+
+  path_bar->ignore_click = FALSE;
+
+  if (widget == path_bar->up_slider_button)
+    gtk_path_bar_scroll_up (path_bar->up_slider_button, path_bar);
+  else if (widget == path_bar->down_slider_button)
+     gtk_path_bar_scroll_down (path_bar->down_slider_button, path_bar);
+
+  if (!path_bar->timer)
+    {
+      path_bar->need_timer = TRUE;
+      path_bar->timer = g_timeout_add (INITIAL_SCROLL_TIMEOUT,
+				       (GSourceFunc)gtk_path_bar_scroll_timeout,
+				       path_bar);
+    }
+
+  return FALSE;
+}
+
+static gboolean
+gtk_path_bar_slider_button_release (GtkWidget      *widget, 
+				    GdkEventButton *event,
+				    GtkPathBar     *path_bar)
+{
+  if (event->type != GDK_BUTTON_RELEASE)
+    return FALSE;
+
+  path_bar->ignore_click = TRUE;
+  gtk_path_bar_stop_scrolling (path_bar);
+
+  return FALSE;
+}
+
+static void
+gtk_path_bar_grab_notify (GtkWidget *widget,
+			  gboolean   was_grabbed)
+{
+  if (!was_grabbed)
+    gtk_path_bar_stop_scrolling (GTK_PATH_BAR (widget));
+}
+
+static void
+gtk_path_bar_state_changed (GtkWidget    *widget,
+			    GtkStateType  previous_state)
+{
+  if (!GTK_WIDGET_IS_SENSITIVE (widget)) 
+    gtk_path_bar_stop_scrolling (GTK_PATH_BAR (widget));
+}
+
+
 /* Changes the icons wherever it is needed */
 static void
 reload_icons (GtkPathBar *path_bar)
@@ -725,17 +887,32 @@ button_clicked_cb (GtkWidget *button,
 		   gpointer   data)
 {
   ButtonData *button_data;
-  GtkWidget *path_bar;
+  GtkPathBar *path_bar;
+  GList *button_list;
+  gboolean child_is_hidden;
 
   button_data = BUTTON_DATA (data);
   if (button_data->ignore_changes)
     return;
 
-  path_bar = button->parent;
-  g_assert (GTK_IS_PATH_BAR (path_bar));
+  path_bar = GTK_PATH_BAR (button->parent);
+
+  button_list = g_list_find (path_bar->button_list, button_data);
+  g_assert (button_list != NULL);
+
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
-  g_signal_emit (path_bar, path_bar_signals [PATH_CLICKED], 0, button_data->path);
+  if (button_list->prev)
+    {
+      ButtonData *child_data;
+
+      child_data = BUTTON_DATA (button_list->prev->data);
+      child_is_hidden = child_data->file_is_hidden;
+    }
+  else
+    child_is_hidden = FALSE;
+
+  g_signal_emit (path_bar, path_bar_signals [PATH_CLICKED], 0, button_data->path, child_is_hidden);
 }
 
 static GdkPixbuf *
@@ -798,19 +975,50 @@ button_data_free (ButtonData *button_data)
   g_free (button_data);
 }
 
+static const char *
+get_dir_name (ButtonData *button_data)
+{
+  if (button_data->type == HOME_BUTTON)
+    return _("Home");
+  else if (button_data->type == DESKTOP_BUTTON)
+    return _("Desktop");
+  else
+    return button_data->dir_name;
+
+}
+
+/* We always want to request the same size for the label, whether
+ * or not the contents are bold
+ */
+static void
+label_size_request_cb (GtkWidget      *widget,
+		       GtkRequisition *requisition,
+		       ButtonData     *button_data)
+{
+  const gchar *dir_name = get_dir_name (button_data);
+  PangoLayout *layout = gtk_widget_create_pango_layout (button_data->label, dir_name);
+  gint bold_width, bold_height;
+  gchar *markup;
+
+  pango_layout_get_pixel_size (layout, &requisition->width, &requisition->height);
+  
+  markup = g_markup_printf_escaped ("<b>%s</b>", dir_name);
+  pango_layout_set_markup (layout, markup, -1);
+  g_free (markup);
+
+  pango_layout_get_pixel_size (layout, &bold_width, &bold_height);
+  requisition->width = MAX (requisition->width, bold_width);
+  requisition->height = MAX (requisition->height, bold_height);
+  
+  g_object_unref (layout);
+}
+
 static void
 gtk_path_bar_update_button_appearance (GtkPathBar *path_bar,
 				       ButtonData *button_data,
 				       gboolean    current_dir)
 {
-  const gchar *dir_name;
-
-  if (button_data->type == HOME_BUTTON)
-    dir_name = _("Home");
-  else if (button_data->type == DESKTOP_BUTTON)
-    dir_name = _("Desktop");
-  else
-    dir_name = button_data->dir_name;
+  const gchar *dir_name = get_dir_name (button_data);
 
   if (button_data->label != NULL)
     {
@@ -847,25 +1055,63 @@ static ButtonType
 find_button_type (GtkPathBar  *path_bar,
 		  GtkFilePath *path)
 {
-  if (! gtk_file_path_compare (path, path_bar->root_path))
+  if (path_bar->root_path != NULL &&
+      ! gtk_file_path_compare (path, path_bar->root_path))
     return ROOT_BUTTON;
-  if (! gtk_file_path_compare (path, path_bar->home_path))
+  if (path_bar->home_path != NULL &&
+      ! gtk_file_path_compare (path, path_bar->home_path))
     return HOME_BUTTON;
-  if (! gtk_file_path_compare (path, path_bar->desktop_path))
+  if (path_bar->desktop_path != NULL &&
+      ! gtk_file_path_compare (path, path_bar->desktop_path))
     return DESKTOP_BUTTON;
 
  return NORMAL_BUTTON;
 }
 
+static void
+button_drag_data_get_cb (GtkWidget          *widget,
+			 GdkDragContext     *context,
+			 GtkSelectionData   *selection_data,
+			 guint               info,
+			 guint               time_,
+			 gpointer            data)
+{
+  ButtonData *button_data;
+  GtkPathBar *path_bar;
+  char *uri;
+  char *uri_list;
+
+  button_data = data;
+  path_bar = GTK_PATH_BAR (widget->parent); /* the button's parent *is* the path bar */
+
+  uri = gtk_file_system_path_to_uri (path_bar->file_system, button_data->path);
+  uri_list = g_strconcat (uri, "\r\n", NULL);
+  g_free (uri);
+
+  gtk_selection_data_set (selection_data,
+			  selection_data->target,
+			  8,
+			  uri_list,
+			  strlen (uri_list));
+  g_free (uri_list);
+}
+
 static ButtonData *
 make_directory_button (GtkPathBar  *path_bar,
 		       const char  *dir_name,
 		       GtkFilePath *path,
-		       gboolean     current_dir)
+		       gboolean     current_dir,
+		       gboolean     file_is_hidden)
 {
+  const GtkTargetEntry targets[] = {
+    { "text/uri-list", 0, 0 }
+  };
+
   GtkWidget *child = NULL;
+  GtkWidget *label_alignment = NULL;
   ButtonData *button_data;
 
+  file_is_hidden = !! file_is_hidden;
   /* Is it a special button? */
   button_data = g_new0 (ButtonData, 1);
 
@@ -883,19 +1129,32 @@ make_directory_button (GtkPathBar  *path_bar,
     case DESKTOP_BUTTON:
       button_data->image = gtk_image_new ();
       button_data->label = gtk_label_new (NULL);
+      label_alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+      gtk_container_add (GTK_CONTAINER (label_alignment), button_data->label);
       child = gtk_hbox_new (FALSE, 2);
       gtk_box_pack_start (GTK_BOX (child), button_data->image, FALSE, FALSE, 0);
-      gtk_box_pack_start (GTK_BOX (child), button_data->label, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (child), label_alignment, FALSE, FALSE, 0);
       break;
     case NORMAL_BUTTON:
     default:
       button_data->label = gtk_label_new (NULL);
-      child = button_data->label;
+      label_alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+      gtk_container_add (GTK_CONTAINER (label_alignment), button_data->label);
+      child = label_alignment;
       button_data->image = NULL;
     }
 
+  /* label_alignment is created because we can't override size-request
+   * on label itself and still have the contents of the label centered
+   * properly in the label's requisition
+   */
+  if (label_alignment)
+    g_signal_connect (label_alignment, "size-request",
+		      G_CALLBACK (label_size_request_cb), button_data);
+
   button_data->dir_name = g_strdup (dir_name);
   button_data->path = gtk_file_path_new_dup (gtk_file_path_get_string (path));
+  button_data->file_is_hidden = file_is_hidden;
 			  
   gtk_container_add (GTK_CONTAINER (button_data->button), child);
   gtk_widget_show_all (button_data->button);
@@ -908,6 +1167,14 @@ make_directory_button (GtkPathBar  *path_bar,
   g_object_weak_ref (G_OBJECT (button_data->button),
 		     (GWeakNotify) button_data_free, button_data);
 
+  gtk_drag_source_set (button_data->button,
+		       GDK_BUTTON1_MASK,
+		       targets,
+		       G_N_ELEMENTS (targets),
+		       GDK_ACTION_COPY);
+  g_signal_connect (button_data->button, "drag-data-get",
+		    G_CALLBACK (button_drag_data_get_cb), button_data);
+
   return button_data;
 }
 
@@ -939,6 +1206,13 @@ gtk_path_bar_check_parent_path (GtkPathBar         *path_bar,
 						 BUTTON_DATA (list->data),
 						 (list == current_path) ? TRUE : FALSE);
 	}
+
+      if (!gtk_widget_get_child_visible (BUTTON_DATA (current_path->data)->button))
+	{
+	  path_bar->first_scrolled_button = current_path;
+	  gtk_widget_queue_resize (GTK_WIDGET (path_bar));
+	}
+
       return TRUE;
     }
   return FALSE;
@@ -974,6 +1248,7 @@ _gtk_path_bar_set_path (GtkPathBar         *path_bar,
       GtkFilePath *parent_path = NULL;
       ButtonData *button_data;
       const gchar *display_name;
+      gboolean is_hidden;
       GtkFileFolder *file_folder;
       GtkFileInfo *file_info;
       gboolean valid;
@@ -991,7 +1266,7 @@ _gtk_path_bar_set_path (GtkPathBar         *path_bar,
 
       file_folder = gtk_file_system_get_folder (path_bar->file_system,
 						parent_path ? parent_path : path,
-						GTK_FILE_INFO_DISPLAY_NAME,
+						GTK_FILE_INFO_DISPLAY_NAME | GTK_FILE_INFO_IS_HIDDEN,
 						NULL);
       if (!file_folder)
 	{
@@ -1013,8 +1288,9 @@ _gtk_path_bar_set_path (GtkPathBar         *path_bar,
 	}
 
       display_name = gtk_file_info_get_display_name (file_info);
+      is_hidden = gtk_file_info_get_is_hidden (file_info);
 
-      button_data = make_directory_button (path_bar, display_name, path, first_directory);
+      button_data = make_directory_button (path_bar, display_name, path, first_directory, is_hidden);
       gtk_file_info_free (file_info);
       gtk_file_path_free (path);
 
@@ -1049,9 +1325,10 @@ _gtk_path_bar_set_path (GtkPathBar         *path_bar,
 
       for (l = new_buttons; l; l = l->next)
 	{
-	  GtkWidget *button = BUTTON_DATA (l->data)->button;
-	  gtk_widget_destroy (button);
-	  gtk_widget_unref (button);
+	  ButtonData *button_data;
+
+	  button_data = BUTTON_DATA (l->data);
+	  gtk_widget_destroy (button_data->button);
 	}
 
       g_list_free (new_buttons);
@@ -1078,11 +1355,22 @@ _gtk_path_bar_set_file_system (GtkPathBar    *path_bar,
   path_bar->file_system = g_object_ref (file_system);
 
   home = g_get_home_dir ();
-  desktop = g_build_filename (home, "Desktop", NULL);
-  path_bar->home_path = gtk_file_system_filename_to_path (path_bar->file_system, home);
-  path_bar->desktop_path = gtk_file_system_filename_to_path (path_bar->file_system, desktop);
+  if (home != NULL)
+    {
+      path_bar->home_path = gtk_file_system_filename_to_path (path_bar->file_system, home);
+      /* FIXME: Need file system backend specific way of getting the
+       * Desktop path.
+       */
+      desktop = g_build_filename (home, "Desktop", NULL);
+      path_bar->desktop_path = gtk_file_system_filename_to_path (path_bar->file_system, desktop);
+      g_free (desktop);
+    }
+  else
+    {
+      path_bar->home_path = NULL;
+      path_bar->desktop_path = NULL;
+    }
   path_bar->root_path = gtk_file_system_filename_to_path (path_bar->file_system, "/");
-  g_free (desktop);
 }
 
 /**
